@@ -6,17 +6,15 @@ use warnings;
 use Carp;
 use Lingua::KO::Hangul::Util 0.06;
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 our $PACKAGE = __PACKAGE__;
 
 require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw( NFC NFD NFKC NFKD );
 our @EXPORT_OK = qw(
-    normalize
-    decompose reorder compose
+    normalize decompose reorder compose
     getCanon getCompat getComposite getCombinClass isExclusion
-    NFC_check NFD_check NFKC_check NFKD_check
 );
 our %EXPORT_TAGS = ( all => [ @EXPORT, @EXPORT_OK ] );
 
@@ -149,16 +147,13 @@ sub decompose {
 }
 
 ##
-## string reorder(string, check?)
+## string reorder(string)
 ##
 sub reorder {
     my @src = unpack('U*', $_[0]);
-    my $chk = $_[1] ? 1 : 0;
-
-    my $pos_last_S = 0;
 
     for(my $i=0; $i < @src;){
-	$pos_last_S = $i++, next if ! $Combin{ $src[$i] };
+	$i++, next if ! $Combin{ $src[$i] };
 
 	my $ini = $i;
 	$i++ while $i < @src && $Combin{ $src[$i] };
@@ -169,15 +164,12 @@ sub reorder {
 
 	@src[ $ini .. $i - 1 ] = @src[ @tmp ];
     }
-    if($chk) {
-	$_[0] = pack 'U*', splice @src, $pos_last_S;
-    }
     return pack('U*', @src);
 }
 
 
 ##
-## string compose(string, check?)
+## string compose(string)
 ##
 ## S : starter; NS : not starter;
 ##
@@ -188,16 +180,11 @@ sub reorder {
 sub compose
 {
     my @src = unpack('U*', $_[0]);
-    my $chk = $_[1] ? 1 : 0;
-
-    my $pos_last_S = 0;
 
     for(my $s = 0; $s+1 < @src; $s++){
 	next unless defined $src[$s] && ! $Combin{ $src[$s] };
 	 # S only; removed or combining are skipped as a starter.
 
-	$pos_last_S = $s;
-
 	my($c, $blocked, $uncomposed_cc);
 	for(my $j = $s+1; $j < @src && !$blocked; $j++){
 	    ($Combin{ $src[$j] } ? $uncomposed_cc : $blocked) = 1;
@@ -219,9 +206,6 @@ sub compose
 	    if($blocked) { $blocked = 0 } else { -- $uncomposed_cc }
 	}
     }
-    if($chk) {
-	$_[0] = pack 'U*', grep defined(), splice @src, $pos_last_S;
-    }
     return pack 'U*', grep defined(), @src;
 }
 
@@ -231,7 +215,6 @@ sub compose
 
 use constant CANON  => 0;
 use constant COMPAT => 1;
-use constant CHECKD => 1;
 
 sub NFD  ($) { reorder(decompose($_[0], CANON )) }
 sub NFKD ($) { reorder(decompose($_[0], COMPAT)) }
@@ -239,36 +222,15 @@ sub NFKD ($) { reorder(decompose($_[0], COMPAT)) }
 sub NFC  ($) { compose(reorder(decompose($_[0], CANON ))) }
 sub NFKC ($) { compose(reorder(decompose($_[0], COMPAT))) }
 
-sub NFD_check ($) {
-    $_[0] = decompose($_[0], CANON );
-    reorder($_[0], CHECKD);
-}
-sub NFKD_check($) {
-    $_[0] = decompose($_[0], COMPAT);
-    reorder($_[0], CHECKD);
-}
-sub NFC_check ($) {
-    $_[0] = reorder(decompose($_[0], CANON ));
-    compose($_[0], CHECKD);
-}
-sub NFKC_check($) {
-    $_[0] = reorder(decompose($_[0], COMPAT));
-    compose($_[0], CHECKD);
-}
-
 sub normalize($$)
 {
     my $form = shift;
-    $form =~ /^NF/;
+    $form =~ s/^NF//;
     return
 	$form eq 'D'  ? NFD ($_[0]) :
 	$form eq 'C'  ? NFC ($_[0]) :
 	$form eq 'KD' ? NFKD($_[0]) :
 	$form eq 'KC' ? NFKC($_[0]) :
-	$form eq 'D_check'  ? NFD_check ($_[0]) :
-	$form eq 'C_check'  ? NFC_check ($_[0]) :
-	$form eq 'KD_check' ? NFKD_check($_[0]) :
-	$form eq 'KC_check' ? NFKC_check($_[0]) :
       croak $PACKAGE."::normalize: invalid form name: $form";
 }
 
@@ -283,3 +245,132 @@ sub _getExclus { wantarray ? %Exclus : \%Exclus }
 
 1;
 __END__
+
+
+=head1 NAME
+
+Unicode::Normalize - normalized forms of Unicode text
+
+=head1 SYNOPSIS
+
+  use Unicode::Normalize;
+
+  $string_NFD  = NFD($raw_string);  # Normalization Form D
+  $string_NFC  = NFC($raw_string);  # Normalization Form C
+  $string_NFKD = NFKD($raw_string); # Normalization Form KD
+  $string_NFKC = NFKC($raw_string); # Normalization Form KC
+
+   or
+
+  use Unicode::Normalize 'normalize';
+
+  $string_NFD  = normalize('D',  $raw_string);  # Normalization Form D
+  $string_NFC  = normalize('C',  $raw_string);  # Normalization Form C
+  $string_NFKD = normalize('KD', $raw_string);  # Normalization Form KD
+  $string_NFKC = normalize('KC', $raw_string);  # Normalization Form KC
+
+=head1 DESCRIPTION
+
+=head2 Normalization
+
+=over 4
+
+=item C<$string_NFD = NFD($raw_string)>
+
+returns the Normalization Form D (formed by canonical decomposition).
+
+
+=item C<$string_NFC = NFC($raw_string)>
+
+returns the Normalization Form C (formed by canonical decomposition
+followed by canonical composition).
+
+=item C<$string_NFKD = NFKD($raw_string)>
+
+returns the Normalization Form KD (formed by compatibility decomposition).
+
+=item C<$string_NFKC = NFKC($raw_string)>
+
+returns the Normalization Form KC (formed by compatibility decomposition
+followed by B<canonical> composition).
+
+=item C<$normalized_string = normalize($form_name, $raw_string)>
+
+As C<$form_name>, one of the following names must be given.
+
+  'C'  or 'NFC'  for Normalization Form C
+  'D'  or 'NFD'  for Normalization Form D
+  'KC' or 'NFKC' for Normalization Form KC
+  'KD' or 'NFKD' for Normalization Form KD
+
+=back
+
+=head2 Character Data
+
+These functions are interface of character data used internally.
+If you want only to get unicode normalization forms, 
+you need not to call them by yourself.
+
+=over 4
+
+=item C<$canonical_decomposed = getCanon($codepoint)>
+
+=item C<$compatibility_decomposed = getCompat($codepoint)>
+
+If the character of the specified codepoint is canonically or 
+compatibility decomposable (including Hangul Syllables),
+returns the B<completely decomposed> string equivalent to it.
+
+If it is not decomposable, returns undef.
+
+=item C<$uv_composite = getComposite($uv_here, $uv_next)>
+
+If the couple of two characters here and next (as codepoints) is composable
+(including Hangul Jamo/Syllables and Exclusions),
+returns the codepoint of the composite.
+
+If they are not composable, returns undef.
+
+=item C<$combining_class = getCombinClass($codepoint)>
+
+Returns the combining class as integer of the character.
+
+=item C<$is_exclusion = isExclusion($codepoint)>
+
+Returns a boolean whether the character of the specified codepoint is
+a composition exclusion.
+
+=back
+
+=head2 EXPORT
+
+C<NFC>, C<NFD>, C<NFKC>, C<NFKD>: by default.
+
+C<normalize> and other some functions: on request.
+
+=head1 AUTHOR
+
+SADAHIRO Tomoyuki, E<lt>SADAHIRO@cpan.orgE<gt>
+
+  http://homepage1.nifty.com/nomenclator/perl/
+
+  Copyright(C) 2001, SADAHIRO Tomoyuki. Japan. All rights reserved.
+
+  This program is free software; you can redistribute it and/or 
+  modify it under the same terms as Perl itself.
+
+=head1 SEE ALSO
+
+=over 4
+
+=item http://www.unicode.org/unicode/reports/tr15/
+
+Unicode Normalization Forms - UAX #15
+
+=item L<Lingua::KO::Hangul::Util>
+
+utility functions for Hangul Syllables
+
+=back
+
+=cut
